/****************************************************************************
*
* (C) 2000 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-240
*  Fax   : (49)-6441-905-245
*
* IPC@CHIP SC12 API 
* ---------------------------------------------------------------------------
* $Header: HWAPI.C, 6, 23.01.2002 14:28:01, Christoph Stoidner$
* ---------------------------------------------------------------------------
* $Log: 
*  6    IPC@CHIP  1.5         23.01.2002 14:28:01  Christoph Stoidner modified DMA
*       Functions GetDmaInfo and StartDma
*  5    IPC@CHIP  1.4         16.01.2002 13:18:50  Christoph Stoidner removed
*       duclicate declared function
*  4    IPC@CHIP  1.3         16.01.2002 10:04:28  Christoph Stoidner add new
*       functions and some comments
*  3    IPC@CHIP  1.2         19.06.2001 12:35:06  Markus Bartat   start: Added
*       comments to the c functions
*  2    IPC@CHIP  1.1         31.05.2001 15:18:23  Christoph Stoidner 
*  1    IPC@CHIP  1.0         14.02.2001 15:09:18  Christoph Stoidner 
* $
*****************************************************************************/

/***************************************************************************/
// Includes
/***************************************************************************/
#include <dos.h>
#include "hwapi.h"

/***************************************************************************/
// Defines
/***************************************************************************/

/***************************************************************************/
// Functions
/***************************************************************************/

/***************************************************************************/
// Enable bus
/***************************************************************************/
void pfe_enable_bus (unsigned mask, unsigned char ale)
{
  // Enable bus mask
  asm mov ah, PFE_ENABLE_DATA_BUS
  asm mov al, ale
  asm mov dx, mask
  asm int PFE_INT
}

/***************************************************************************/
// Enable non multplexed address bus
/***************************************************************************/
void pfe_enable_adr (unsigned char adr)
{
  //enable non multiplexed io-bus
  asm mov ah, PFE_ENABLE_NON_MULTI_ADDR_BUS;
  asm mov dx, 1
  asm mov cl, adr
  asm shl dx, cl    ;//dx = 1<<adr
  asm int PFE_INT
}

/***************************************************************************/
// Enable programmable chip selects
/***************************************************************************/
void pfe_enable_pcs (unsigned char pcs)
{
  // Enable programmable chip selects
  asm mov ah, PFE_ENABLE_PCS;
  asm mov dx, 1
  asm mov cl, pcs
  asm shl dx, cl    ;//dx = 1<<pcs
  asm int PFE_INT
}

/***************************************************************************/
// Enable interrupts
/***************************************************************************/
void pfe_enable_int (unsigned char irq)
{
  // Enable interrupts
  asm mov ah, PFE_ENABLE_EXTERNAL_IRQ;
  asm mov dx, 1
  asm mov cl, irq
  asm shl dx, cl    ;//dx= 1<<irq
  asm int PFE_INT
}

/***************************************************************************/
// Enable timer
/***************************************************************************/
void pfe_enable_timer (unsigned mode)
{
  // Enable timer
  asm mov ah, PFE_ENABLE_EXTERNAL_TIMER;
  asm mov dx, mode
  asm int PFE_INT
}

/***************************************************************************/
// Set Edge Level Interrupt Mode
/***************************************************************************/
void pfe_set_edge_level_intr_mode(unsigned char mode, unsigned mask)
{
  // set edge level interrupt mode
  asm mov ah, PFE_SET_EDGE_LEVEL_INTR_MODE;
  asm mov al, mode
  asm mov dx, mask
  asm int PFE_INT
}


/***************************************************************************/
// Enable Pwd Mode
/***************************************************************************/
void pfe_enable_pwd(void)
{
  // enable pwd
  asm mov ah, PFE_ENABLE_PWD;
  asm int PFE_INT
}

/***************************************************************************/
// Enable External Dma
/***************************************************************************/
int pfe_enable_external_dma( unsigned char channel)
{
  int retval;

  // enable pwd
  asm mov ah, PFE_ENABLE_EXTERNAL_DMA;
  asm mov al, channel
  asm int PFE_INT
  asm mov retval, ax
  return retval;
}


/***************************************************************************/
// Enable INT0/INTA cascade mode
/***************************************************************************/
void pfe_enable_int_cascade_mode(void)
{
  // enable int0/inta cascade mode
  asm mov ah, PFE_ENABLE_INT_CASCADE_MODE;
  asm int PFE_INT
}



/***************************************************************************/
// Enable programmable pio pins
/***************************************************************************/
void pfe_enable_pio (unsigned pio, unsigned char mode)
{
  pio = 1 << pio;

  // Enable programmable pios
  asm mov ah, PFE_ENABLE_PIO;
  asm mov al, mode
  asm mov dx, pio
  asm int PFE_INT
}

/***************************************************************************/
// Get programmable pio pins state
/***************************************************************************/
void pfe_get_pio_state (unsigned *wPio, unsigned *wInp, unsigned *wOut)
{
  unsigned p, i, o;

  asm mov ah, PFE_ENABLE_PIO;
  asm mov al, 0
  asm int PFE_INT
  asm mov p, ax
  asm mov i, dx
  asm mov o, cx

  *wPio = p;
  *wInp = i;
  *wOut = o;
}

/***************************************************************************/
// Get programmable pio pins output state
/***************************************************************************/
void hal_get_pio_output_state (unsigned *pins, unsigned wOut)
{
  unsigned p;

  // Read pio outputs init state
  asm mov ah, PFE_ENABLE_PIO;
  asm mov bx, 0xFFFF
  asm xor cx, cx        ;// cx = 0
  asm int HAL_INT
  asm and ax, wOut
  asm mov p, ax

  *pins = p; //*pins = ax & wOut;
}

/***************************************************************************/
// Read databus
/***************************************************************************/
unsigned char hal_read_bus (unsigned adr, unsigned wAND, unsigned wXOR)
{
  unsigned char retval;

  // read data bus
  asm mov ah, HAL_READ_DATA_BUS;
  asm mov di, adr
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
  asm mov retval, al

  return retval;
}

/***************************************************************************/
// Write databus
/***************************************************************************/
void hal_write_bus (unsigned adr, unsigned char value, unsigned wAND, unsigned wXOR)
{
  // write data bus
  asm mov ah, HAL_WRITE_DATA_BUS;
  asm mov di, adr
  asm mov dh, 0
  asm mov dl, value
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
}

/***************************************************************************/
// Read block from databus
/***************************************************************************/
void hal_readblock_bus (unsigned adr0, unsigned adr1, unsigned char *buf, unsigned cnt)
{
  // read data bus
  _ES = FP_SEG(buf);
  _BX = FP_OFF(buf);
  asm mov ah, HAL_BLOCK_READ_DATA_BUS;
  asm mov di, adr0
  asm mov si, adr1
  asm mov cx, cnt
  asm int HAL_INT
}

/***************************************************************************/
// Write block to databus
/***************************************************************************/
void hal_writeblock_bus (unsigned adr0, unsigned adr1, unsigned char *buf, unsigned cnt)
{
  // write data bus
  _ES = FP_SEG(buf);
  _BX = FP_OFF(buf);
  asm mov ah, HAL_BLOCK_WRITE_DATA_BUS;
  asm mov di, adr0
  asm mov si, adr1
  asm mov cx, cnt
  asm int HAL_INT
}

/***************************************************************************/
// Read pios
/***************************************************************************/
unsigned hal_read_pios (unsigned wAND, unsigned wXOR)
{
  unsigned retval;

  // read pios
  asm mov ah, HAL_READ_PIO
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
  asm mov retval, ax

  return retval;
}

/***************************************************************************/
// Write pios
/***************************************************************************/
void hal_write_pios (unsigned value, unsigned wAND, unsigned wXOR)
{
  // write pios
  asm mov ah, HAL_WRITE_PIO
  asm mov dx, value
  asm mov bx, wAND
  asm mov cx, wXOR
  asm int HAL_INT
}

/***************************************************************************/
// Read single pio
/***************************************************************************/
unsigned char hal_read_pio (unsigned char pio)
{
  unsigned retval;

  asm mov ah, HAL_READ_SPECIFIC_PIO
  asm mov al, pio
  asm int HAL_INT
  asm mov retval, ax

  if (retval) return 1;
  else        return 0;
}

/***************************************************************************/
// Write single pio
/***************************************************************************/
void hal_write_pio (unsigned char pio, unsigned value)
{
  asm mov ah, HAL_WRITE_SPECIFIC_PIO
  asm mov al, pio
  asm mov dx, value
  asm int HAL_INT
}

/***************************************************************************/
// Install ISR
/***************************************************************************/
InterruptHandler hal_install_isr (unsigned irq, unsigned count, InterruptHandler handler)
{
  InterruptHandler oldhandler;
  unsigned rbx, res;

  // install isr
  asm les bx, handler  ;// es = FP_SEG(handler);  bx = FP_OFF(handler);
  asm mov ah, HAL_INSTALL_ISR;
  asm mov dx, irq
  asm mov cx, count
  asm int HAL_INT
  asm mov rbx, bx
  asm mov res, es

  oldhandler = (InterruptHandler) MK_FP(res,rbx);

  return oldhandler;
}

/***************************************************************************/
// Init timer
/***************************************************************************/
void hal_init_timer (unsigned char timer, unsigned mode, unsigned divider)
{
  // install isr
  asm mov ah, HAL_INIT_TIMER;
  asm mov al, timer
  asm mov dx, mode
  asm mov cx, divider
  asm int HAL_INT
}

/***************************************************************************/
// Start timer
/***************************************************************************/
void hal_start_timer (unsigned char timer)
{
  // start timer
  asm mov ah, HAL_START_TIMER;
  asm mov al, timer
  asm int HAL_INT
}

/***************************************************************************/
// Stop timer
/***************************************************************************/
void hal_stop_timer (unsigned char timer)
{
  // stop timer
  asm mov ah, HAL_STOP_TIMER;
  asm mov al, timer
  asm int HAL_INT
}

/***************************************************************************/
// Read timer count
/***************************************************************************/
unsigned hal_read_timer_count (unsigned char timer, unsigned far *maxcount)
{
  unsigned retval, count;

  // read count
  asm mov ah, HAL_READ_TIMER_COUNT;
  asm mov al, timer
  asm int HAL_INT
  asm mov count, dx
  asm mov retval, ax

  *maxcount = count;
  return retval;
}

/***************************************************************************/
// Write timer count
/***************************************************************************/
void hal_write_timer_count (unsigned char timer, unsigned counter)
{
  // write count
  asm mov ah, HAL_WRITE_TIMER_COUNT;
  asm mov al, timer
  asm mov dx, counter
  asm int HAL_INT
}

/***************************************************************************/
// Set watchdog mode
/***************************************************************************/
unsigned char hal_set_watchdog_mode (unsigned char mode)
{
  unsigned char retval;

  // set watchdog mode
  asm mov ah, HAL_GET_SET_WATCHDOG_MODE;
  asm mov al, mode
  asm int HAL_INT
  asm mov retval, al

  return retval;
}

/***************************************************************************/
// Refresh watchdog
/***************************************************************************/
void hal_refresh_watchdog (void)
{
  // refresh watchdog
  asm mov ah, HAL_REFRESH_WATCHDOG;
  asm int HAL_INT
}


/***************************************************************************/
// Get Frequencies
/***************************************************************************/
unsigned long hal_get_frequencies( unsigned char frq )
{
  unsigned hword, lword;

  // get frequencies
  asm mov ah, HAL_GET_FREQUENCIES;
  asm mov al, frq
  asm int HAL_INT

  asm mov hword, dx
  asm mov lword, ax

  return (hword<<16)+lword;
}


/***************************************************************************/
// Set Duty Cycle Waveform
/***************************************************************************/
void hal_set_duty_cycle_waveform( unsigned char timer, int mode,
                                  unsigned clock_divider )
{
  // ser dury cycle waveform
  asm mov ah, HAL_SET_DUTY_CYCLE_WAVEFORM;
  asm mov al, timer
  asm mov dx, mode
  asm mov cx, clock_divider
  asm int HAL_INT

  return;
}

/***************************************************************************/
// Give End of Interrupt (EOI)
/***************************************************************************/
void hal_give_eoi( unsigned char intr_no )
{
  // give EOI
  asm mov ah, HAL_GIVE_EOI;
  asm mov al, intr_no
  asm int HAL_INT
  return;
}


/***************************************************************************/
// Start DMA
/***************************************************************************/
void hal_start_dma( unsigned char channel, Dma_Info *dmaInfo )
{
  unsigned long src;
  unsigned long dest;
  struct SREGS segregs;
  union REGS regs;

  // read Source Address
  if ( dmaInfo->ctrl_reg&DMA_SRCMEM )
  {
    src  = FP_SEG(dmaInfo->src.mem);
    src *= 0x10;
    src += FP_OFF(dmaInfo->src.mem);
  }
  else src = dmaInfo->src.io;

  // read dest Address
  if ( dmaInfo->ctrl_reg&DMA_DESTMEM )
  {
     dest  = FP_SEG(dmaInfo->dest.mem);
     dest *= 0x10;
     dest += FP_OFF(dmaInfo->dest.mem);
  }
  else dest = dmaInfo->dest.io;

  // start DMA
  regs.h.ah = HAL_START_DMA;
  regs.h.al = channel;
  regs.x.cx = dmaInfo->count;
  regs.x.dx = dmaInfo->ctrl_reg;
  regs.x.bx = FP_SEG(&src);
  regs.x.si = FP_OFF(&src);
  segregs.es  = FP_SEG(&dest);
  regs.x.di = FP_OFF(&dest);
  int86x( HAL_INT, &regs, &regs, &segregs );
  return;
}

/***************************************************************************/
// Stop DMA
/***************************************************************************/
int hal_stop_dma( unsigned char channel )
{
  int retval;

  asm mov ah, HAL_STOP_DMA
  asm mov al, channel
  asm int HAL_INT
  asm mov retval, ax
  return retval;
}


/***************************************************************************/
// Get DMA Info
/***************************************************************************/
int hal_get_dma_info( unsigned char channel, Dma_Info *dmaInfo )
{
  unsigned long src;
  unsigned long dest;
  unsigned ioff, iseg;
  struct SREGS segregs;
  union REGS regs;

  // start DMA
  regs.h.ah = HAL_GET_DMA_INFO;
  regs.h.al = channel;
  regs.x.bx = FP_SEG(&src);
  regs.x.si = FP_OFF(&src);
  segregs.es  = FP_SEG(&dest);
  regs.x.di = FP_OFF(&dest);
  int86x( HAL_INT, &regs, &regs, &segregs );

  dmaInfo->ctrl_reg = regs.x.dx;
  dmaInfo->count    = regs.x.cx;

  // read dest Address
  if ( dmaInfo->ctrl_reg&DMA_SRCMEM )
  {
     iseg = (unsigned)(src/0x10);
     ioff = (unsigned)(src-((unsigned long)iseg*0x10));
     dmaInfo->src.mem  = MK_FP( iseg, ioff );
  }
  else dmaInfo->src.io = (unsigned)src;

  // read dest Address
  if ( dmaInfo->ctrl_reg&DMA_DESTMEM )
  {
     iseg = (unsigned)(src/0x10);
     ioff = (unsigned)(src-((unsigned long)iseg*0x10));
     dmaInfo->dest.mem  = MK_FP( iseg, ioff );
  }
  else dmaInfo->dest.io = (unsigned)dest;

  return regs.x.ax;
}


/***************************************************************************/
// Init/Restore Non Volatile Data
/***************************************************************************/
int hal_init_restore_non_volatile_data( Remop *remop )
{
   int retval;

   asm les bx, remop
   asm mov ah, HAL_INIT_REST_NON_VOLATILE_DATA
   asm int HAL_INT
   asm mov retval, ax
   return retval;
}


/***************************************************************************/
// Save Non Volatile Data
/***************************************************************************/
void hal_save_non_volatile_data(void)
{
   asm mov ah, HAL_SAVE_NON_VOLATILE_DATA
   asm int HAL_INT
}


/***************************************************************************/
// Get Reboot Reason
/***************************************************************************/
int hal_get_reboot_reason(void)
{
   int retval;

   asm mov ah, HAL_GET_REBOOT_REASON
   asm int HAL_INT
   asm mov retval, ax
   return retval;
}







