/****************************************************************************
*
* (C) 2000 by PWR Solutions GbR
*
*  PWR Solutions GbR
*  Euckenstrasse 17
*  D-81369 Mnchen
*
*  Phone : (49)-89-726092-30
*  Fax   : (49)-89-726092-59
*
* ---------------------------------------------------------------------------
* Module        : I2C_API.C
* Function      : I2C API
*
* Compiler      : Borland C++ 5.0
* Memorymodel   : Large
*
* Author        : Bernhard Roth
* Date          : 17.07.00
* Version       : V1.00
* ---------------------------------------------------------------------------

$Header: i2c_api.c, 6, 16.01.2002 10:45:04, Christoph Stoidner$


$Log: 
 6    IPC@CHIP  1.5         16.01.2002 10:45:04  Christoph Stoidner add starteam
      directives
 5    IPC@CHIP  1.4         16.01.2002 10:11:07  Christoph Stoidner add starteam
      directives
 4    IPC@CHIP  1.3         16.01.2002 10:09:08  Christoph Stoidner add new
      functions and comments
 3    IPC@CHIP  1.2         13.08.2001 16:06:36  Christoph Stoidner Edit header
 2    IPC@CHIP  1.1         13.08.2001 15:29:06  Christoph Stoidner edit
 1    IPC@CHIP  1.0         13.08.2001 15:26:59  Christoph Stoidner 
$

*
* History       :
*
*  Vx.yy                   Author                Changes
*
*  V1.00      17.07.00     roth                  Create
*
*  -------------------------------------------------------------------------
*  Published on the Beck WebSite with kindlier permission of Bernhard Roth.
****************************************************************************/



/****************************************************************************/
//includes
/****************************************************************************/

#include <dos.h>
#include <stdio.h>
#include "I2C_API.H"


/*************************************************************************/
//Init the I2C Interface
/*************************************************************************/
void I2C_init (void)
{
  union  REGS  inregs;
  union  REGS  outregs;


  inregs.h.ah = I2C_INIT;

  int86(I2CINT,&inregs,&outregs);

}

/*************************************************************************/
//Release I2C bus
/*************************************************************************/
void I2C_release (void)
{
  union  REGS  inregs;
  union  REGS  outregs;


  inregs.h.ah = I2C_RELEASE;
  inregs.h.al = 0;
  int86(I2CINT,&inregs,&outregs);

}

/*************************************************************************/
//Restart I2C
/*************************************************************************/
void I2C_restart (void)
{
 union  REGS  inregs;
 union  REGS  outregs;

 inregs.h.ah = I2C_RESTART;
 inregs.h.al = 0;
 int86(I2CINT,&inregs,&outregs);

}


/*************************************************************************/
//Scan for interfaces on I2C
/*************************************************************************/
unsigned char I2C_scan (unsigned char start_addr, unsigned char end_addr)
{
 union  REGS  inregs;
 union  REGS  outregs;


 inregs.h.ah = I2C_SCAN;
 inregs.h.al = start_addr;
 inregs.h.cl = end_addr;
 int86(I2CINT,&inregs,&outregs);

 return outregs.h.al;

}

/*************************************************************************/
//Transmit Block
/*************************************************************************/
int I2C_transmit_block (unsigned char slave, char far * buffer, int length)
{
 union  REGS  inregs;
 union  REGS  outregs;
 struct SREGS sregs;

 inregs.h.ah = I2C_TRANS_RECV_BLOCK;
 inregs.h.al = slave & 0xFE;
 inregs.x.cx = length;
 sregs.es    = FP_SEG(buffer);
 inregs.x.bx = FP_OFF(buffer);
 int86x(I2CINT,&inregs,&outregs,&sregs);

 if (outregs.x.flags & 0x01)
 {
  return outregs.h.al;
 }

 return 0;

}

/*************************************************************************/
//Receive Block
/*************************************************************************/
int I2C_receive_block (unsigned char slave, char far * buffer, int length)
{
 union  REGS  inregs;
 union  REGS  outregs;
 struct SREGS sregs;


 inregs.h.ah = I2C_TRANS_RECV_BLOCK;
 inregs.h.al = slave | 0x01;
 inregs.x.cx = length;
 sregs.es    = FP_SEG(buffer);
 inregs.x.bx = FP_OFF(buffer);
 int86x(I2CINT,&inregs,&outregs,&sregs);

 if (outregs.x.flags & 0x01)
 {
  return outregs.h.al;
 }

 return 0;

}

/*************************************************************************/
//Transmit one character
/*************************************************************************/
int I2C_transmit_char (unsigned char slave, char c)
{
 union  REGS  inregs;
 union  REGS  outregs;

 inregs.h.ah = I2C_TRANS_RECV_CHAR;
 inregs.h.al = slave & 0xFE;
 inregs.h.cl = c;
 int86(I2CINT,&inregs,&outregs);

 if (outregs.x.flags & 0x01)
 {
  return (int)outregs.h.al & 0x00FF;
 }

 return 0;
}

/*************************************************************************/
//Receive one Character
/*************************************************************************/
int I2C_receive_char (unsigned char slave, char * c, unsigned char lastchar)
{
 union  REGS  inregs;
 union  REGS  outregs;

 inregs.h.ah = I2C_TRANS_RECV_CHAR;
 inregs.h.al = slave | 0x01;
 if (lastchar)
 {
  inregs.h.cl = 1;     //want more characters
 }
 else
 {
  inregs.h.cl = 0;     //only one character
 }
 int86(I2CINT,&inregs,&outregs);

 if (outregs.x.flags & 0x01)
 {
  *c = 0;
  return (int)outregs.h.al & 0x00FF;
 }

 *c = (char)outregs.h.ch;
 return 0;
}

/*************************************************************************/
//Select I2C Clock Pin
/*************************************************************************/
void I2C_select_clock_pin(unsigned char pio_no)
{
 union  REGS  inregs;
 union  REGS  outregs;

 inregs.h.ah = I2C_SELECT_CLK_PIN;
 inregs.h.al = pio_no;
 int86(I2CINT,&inregs,&outregs);
}

/*************************************************************************/
//Select I2C data Pin
/*************************************************************************/
void I2C_select_data_pin(unsigned char pio_no)
{
 union  REGS  inregs;
 union  REGS  outregs;

 inregs.h.ah = I2C_SELECT_DATA_PIN;
 inregs.h.al = pio_no;
 int86(I2CINT,&inregs,&outregs);
}
